import React, { FC } from 'react';
import {
    BodyText,
    Stack,
    Button,
    ButtonGroup,
    Link,
    Checkbox,
    Toast,
    Frame,
    Headline,
    Page,
    Layout,
    Card,
} from '@servicetitan/design-system';
import { FeaturePopover } from './components/feature-popover';
import { FeatureModal } from './components/feature-modal';
import { FeatureToast } from './components/feature-toast';
import { TourPopover } from './components/tour-popover';
import geofence from '../../../storybook-static/geofence.png';
import dashboard from '../../../storybook-static/dashboard.png';

import { TourProvider, StepType, useTour } from '@reactour/tour';

const image = {
    src: geofence,
    alt: 'alt text description',
};

const imageDashboard = {
    src: dashboard,
    alt: 'alt text description',
};

export default {
    title: 'Feature Spotlight/Demos',
};

export const Popover = () => (
    <React.Fragment>
        <Stack
            direction="column"
            spacing={5}
            alignItems="center"
            justifyContent="center"
            className="p-4"
            style={{ height: '800px' }}
        >
            <FeaturePopover
                trigger={<Button inactive>Top</Button>}
                open
                direction="t"
                header="👋 Bills has moved"
                footer={
                    <ButtonGroup>
                        <Button small primary>
                            Take me there
                        </Button>
                    </ButtonGroup>
                }
                footerAlign="right"
                offset={{ distance: 200 }}
            >
                <Stack spacing={1} direction="column">
                    <BodyText size="small">
                        Bills has a new home in Accounting. It is still the same feature you know
                        and love except now something goes here.
                        <Link primary>Learn More</Link>
                    </BodyText>
                    <Checkbox size="small" label="Don’t show me again" />
                </Stack>
            </FeaturePopover>

            <FeaturePopover
                trigger={<Button inactive>Right</Button>}
                open
                direction="r"
                header="👋 Bills has moved"
                footer={
                    <ButtonGroup>
                        <Button small primary>
                            Take me there
                        </Button>
                    </ButtonGroup>
                }
                footerAlign="right"
                offset={{ distance: 200 }}
            >
                <Stack spacing={1} direction="column">
                    <BodyText size="small">
                        Bills has a new home in Accounting. It is still the same feature you know
                        and love except now something goes here.
                        <Link primary>Learn More</Link>
                    </BodyText>
                    <Checkbox size="small" label="Don’t show me again" />
                </Stack>
            </FeaturePopover>

            <FeaturePopover
                trigger={<Button inactive>Left</Button>}
                open
                direction="l"
                header="👋 Bills has moved"
                footer={
                    <ButtonGroup>
                        <Button small primary>
                            Take me there
                        </Button>
                    </ButtonGroup>
                }
                footerAlign="right"
                offset={{ distance: 200 }}
            >
                <Stack spacing={1} direction="column">
                    <BodyText size="small">
                        Bills has a new home in Accounting. It is still the same feature you know
                        and love except now something goes here.
                        <Link primary>Learn More</Link>
                    </BodyText>
                    <Checkbox size="small" label="Don’t show me again" />
                </Stack>
            </FeaturePopover>

            <FeaturePopover
                trigger={<Button inactive>Bottom</Button>}
                open
                direction="b"
                header="👋 Bills has moved"
                footer={
                    <ButtonGroup>
                        <Button small primary>
                            Take me there
                        </Button>
                    </ButtonGroup>
                }
                footerAlign="right"
                offset={{ distance: 200 }}
            >
                <Stack spacing={1} direction="column">
                    <BodyText size="small">
                        Bills has a new home in Accounting. It is still the same feature you know
                        and love except now something goes here.
                        <Link primary>Learn More</Link>
                    </BodyText>
                    <Checkbox size="small" label="Don’t show me again" />
                </Stack>
            </FeaturePopover>
        </Stack>
    </React.Fragment>
);

const closeMethod = () => alert('closing');

export const modal = (args: any) => <FeatureModal {...args} />;
modal.args = {
    open: true,
    title: 'FleetPro Has Launched!',
    closable: true,
    onClose: closeMethod,
    scrollable: true,
    children: (
        <Stack direction="column" alignItems="center" spacing={4}>
            <img src={image.src} alt={image.alt} className="p-t-4" />
            <BodyText size="large" className="ta-center">
                We’re excited to announce that FleetPro is now live in ServiceTitan! With FleetPro,
                you can manage and track your company’s vehicles all in one place. Add FleetPro to
                your existing tenant now for $100 /mo or view the{' '}
                <Link primary>Knowledge Base</Link> to learn more.
            </BodyText>
        </Stack>
    ),
    footer: (
        <ButtonGroup>
            <Button onClick={closeMethod}>No, Thank You</Button>
            <Button primary onClick={closeMethod}>
                Add FleetPro Now
            </Button>
        </ButtonGroup>
    ),
};

export const toast = () => {
    return (
        <Frame>
            <Toast.Group>
                <FeatureToast
                    title="✨ This page has a new look and feel"
                    onPrimaryActionClick={() => {}}
                    primaryActionName="Got it"
                    onSecondaryActionClick={() => {}}
                    secondaryActionName="Revert to Old View"
                    onClose={closeMethod}
                    portal={false}
                >
                    <BodyText>
                        We’ve updated this page with some cool stuff...but if you don’t like it you
                        can go back to the old one for now. This can be changed later in settings.
                    </BodyText>
                </FeatureToast>
                <Toast status="info" title="Ordinary toast" portal={false}>
                    In progress
                </Toast>
                <FeatureToast
                    center
                    title="Dashboards"
                    onPrimaryActionClick={() => {}}
                    primaryActionName="Learn more"
                    onClose={closeMethod}
                    portal={false}
                >
                    <Stack direction="column" spacing={2} alignItems="center">
                        <img
                            src={imageDashboard.src}
                            alt={image.alt}
                            width="166px"
                            className="p-t-1"
                        />
                        <BodyText>
                            We noticed you don’t really use this thing, but did you know it could
                            increase your profit margins by 10%?
                        </BodyText>
                    </Stack>
                </FeatureToast>
                <FeatureToast
                    center
                    title="Slightly longer line that wraps to a second"
                    onPrimaryActionClick={() => {}}
                    primaryActionName="Learn More"
                    onClose={closeMethod}
                    portal={false}
                >
                    <Stack direction="column" spacing={2} alignItems="center">
                        <img
                            src={imageDashboard.src}
                            alt={image.alt}
                            width="166px"
                            className="p-t-1"
                        />
                        <BodyText>
                            We noticed you don’t really use this thing, but did you know it could
                            increase your profit margins by 10%?
                        </BodyText>
                    </Stack>
                </FeatureToast>
            </Toast.Group>
        </Frame>
    );
};

interface InternalTourProps {
    children?: React.ReactNode;
    headerText?: string;
}

const InternalTourControls: FC<InternalTourProps> = React.memo(({ children, headerText }) => {
    const { setCurrentStep, currentStep, steps, setIsOpen } = useTour();

    const handleClose = React.useCallback(() => {
        setIsOpen(false);
    }, [setIsOpen]);

    const handleNext = React.useCallback(() => {
        if (currentStep === steps.length - 1) {
            handleClose();
        } else {
            setCurrentStep(currentStep + 1);
        }
    }, [currentStep, handleClose, setCurrentStep, steps.length]);

    return (
        <TourPopover
            currentStep={currentStep}
            onClose={handleClose}
            onNext={handleNext}
            totalStepCount={steps.length}
            headerText={headerText}
        >
            {children}
        </TourPopover>
    );
});

const ReactTour: FC = ({ children }) => {
    const steps: StepType[] = [
        {
            selector: '.step-1',
            content: (
                <InternalTourControls headerText="⚠️ This is now a required field">
                    <BodyText size="small" className="m-b-2">
                        You must now select a date range beforehand because of reasons.
                    </BodyText>
                    <BodyText size="small">
                        <Link primary>Learn More</Link>
                    </BodyText>
                </InternalTourControls>
            ),
        },
        {
            selector: '.step-2',
            content: (
                <InternalTourControls headerText="Give it up for step 2"></InternalTourControls>
            ),
        },
        {
            selector: '.step-3',
            content: (
                <InternalTourControls>
                    This is the final step with just body text.
                </InternalTourControls>
            ),
        },
    ];

    const styles = {
        popover: (base: any) => ({
            ...base,
            borderRadius: '6px',
            padding: 'inherit',
            margin: 'inherit',
            maxWidth: 'inherit',
        }),
    };

    return (
        <TourProvider
            defaultOpen
            steps={steps}
            showBadge={false}
            showPrevNextButtons={false}
            showCloseButton={false}
            showNavigation={false}
            showDots={false}
            styles={styles}
        >
            {children}
        </TourProvider>
    );
};

const StartTourButton = () => {
    const { setIsOpen, setCurrentStep } = useTour();

    const handleTourClick = React.useCallback(() => {
        setCurrentStep(0);
        setIsOpen(true);
    }, [setCurrentStep, setIsOpen]);

    return <Button onClick={handleTourClick}>Start Tour</Button>;
};

export const tour = () => {
    return (
        <Frame>
             <ReactTour>
                <Page
                    header={
                        <Stack justifyContent="space-between" alignItems="center">
                            <Headline size="large" className="step-3">Page Title</Headline>
                            <StartTourButton />
                        </Stack>
                    }
                >
                    <Layout type="3Col">
                        <Layout.Section>
                            <Card className="step-1">Card 1</Card>
                        </Layout.Section>
                        <Layout.Section>
                            <Card className="step-2">Card 2</Card>
                        </Layout.Section>
                        <Layout.Section>
                            <Card>Card 3</Card>
                        </Layout.Section>
                    </Layout>
                </Page>
            </ReactTour>
        </Frame>
    );
};
